import React from 'react';
import PropTypes from 'prop-types';
import { injectStylePrefixed } from 'styletron-utils';

/**
 * @class StyletronProvider
 * @packagename styletron-react
 * @description Provides a Styletron instance to descendant styled components via context
 * @example
 * const Styletron = require('styletron');
 *
 * function render() {
 *   return React.renderToString(
 *     <StyletronProvider styletron={new Styletron()}>
 *       <App/>
 *     </StyletronProvider>
 *   );
 * }
 *
 * @property {object} styletron - Styletron instance
 * @property {ReactElement} children - children
 * @extends ReactClass
 */

class StyletronProvider extends React.Component {
  getChildContext() {
    return {
      styletron: this.styletron
    };
  }

  constructor(props, context) {
    super(props, context);
    this.styletron = props.styletron;
  }

  render() {
    return React.Children.only(this.props.children);
  }

}

StyletronProvider.propTypes = {
  styletron: PropTypes.object.isRequired,
  children: PropTypes.element.isRequired
};
StyletronProvider.childContextTypes = {
  styletron: PropTypes.object.isRequired
};

const STYLETRON_KEY = '__STYLETRON';
/**
 * Helper function to create styled element components
 * @packagename styletron-react
 * @param  {String|function} base        Tag name or component
 * @param  {function|object} styleFn     Style object or function that returns a style object
 * @param  {function}        assignProps Function that consumes the style result and props and returns an object with new props
 * @return {function}                    Component
 * @example
 */

function core(base, style, assignProps) {
  if (typeof base === 'function' && base[STYLETRON_KEY]) {
    const {
      tag,
      styles
    } = base[STYLETRON_KEY]; // Styled component

    return createStyledElementComponent(tag, styles.concat(style), assignProps);
  }

  if (typeof base === 'string' || typeof base === 'function') {
    // Tag name or non-styled component
    return createStyledElementComponent(base, [style], assignProps);
  }

  throw new Error('`styled` takes either a DOM element name or a component');
}

function createStyledElementComponent(base, stylesArray, assignProps) {
  function StyledElement(props, context) {
    const ownProps = assign({}, props);
    delete ownProps.innerRef;
    const styleResult = {};
    StyledElement[STYLETRON_KEY].styles.forEach(style => {
      if (typeof style === 'function') {
        assign(styleResult, style(ownProps, context));
      } else if (typeof style === 'object') {
        assign(styleResult, style);
      }
    });
    let elementProps = assignProps(context.styletron, styleResult, ownProps);
    elementProps = omit$Props(elementProps);

    if (props.innerRef) {
      elementProps.ref = props.innerRef;
    }

    return React.createElement(StyledElement[STYLETRON_KEY].tag, elementProps);
  }

  StyledElement[STYLETRON_KEY] = {
    tag: base,
    styles: stylesArray
  };
  StyledElement.contextTypes = {
    styletron: PropTypes.object
  };

  if (process.env.NODE_ENV !== "production") {
    const name = base.displayName ? base.displayName : typeof base === 'function' ? base.name : base;
    StyledElement.displayName = `Styled${name ? `(${name})` : ''}`;
  }

  return StyledElement;
}

function assign(target, source) {
  for (const key in source) {
    target[key] = source[key];
  }

  return target;
}

function omit$Props(source) {
  const result = {};

  for (const key in source) {
    if (key[0] !== '$') {
      result[key] = source[key];
    }
  }

  return result;
}

/**
 * Helper function to create styled element components
 * @packagename styletron-react
 * @param  {String|function} base     Tag name or styled element component
 * @param  {function|object} styleFn  Style object or function that returns a style object
 * @return {function}                 Styled element component
 * @example
 * import {styled} from 'styletron-react';
 *
 * const Panel = styled('div', {
 *   backgroundColor: 'lightblue',
 *   fontSize: '12px'
 * });
 *
 * <Panel>Hello World</Panel>
 * @example
 * import {styled} from 'styletron-react';
 *
 * const Panel = styled('div', (props) => ({
 *   backgroundColor: props.alert ? 'orange' : 'lightblue',
 *   fontSize: '12px'
 * }));
 *
 * <Panel alert>Danger!</Panel>
 * @example
 * import {styled} from 'styletron-react';
 *
 * const DeluxePanel = styled(Panel, (props) => ({
 *   backgroundColor: props.alert ? 'firebrick' : 'rebeccapurple',
 *   color: 'white',
 *   boxShadow: '3px 3px 3px darkgray'
 * }));
 *
 * <DeluxePanel>Bonjour Monde</DeluxePanel>
 */

function styled(base, style) {
  return core(base, style, assignProps);
}

function assignProps(styletron, styleResult, ownProps) {
  const styletronClassName = injectStylePrefixed(styletron, styleResult); // Skipping cloning of `ownProps` since that's already done internally

  if (ownProps.styleProps) {
    delete ownProps.styleProps;
  }

  ownProps.className = ownProps.className ? `${ownProps.className} ${styletronClassName}` : styletronClassName;
  return ownProps;
}

export { StyletronProvider, core, styled };
//# sourceMappingURL=browser.es2017.es.js.map
